@@ -1,9 +1,9 @@
-#!/usr/local/bin/python2.1
+#!/usr/local/bin/python
 
 """ Configuration for the eGenix mx Base Distribution
 
     Copyright (c) 1997-2000, Marc-Andre Lemburg; mailto:mal@lemburg.com
-    Copyright (c) 2000-2014, eGenix.com Software GmbH; mailto:info@egenix.com
+    Copyright (c) 2000-2015, eGenix.com Software GmbH; mailto:info@egenix.com
     See the documentation for further information on copyrights,
     or contact the author. All Rights Reserved.
 """
@@ -13,7 +13,9 @@ from mxSetup import mx_Extension, mx_version
 #
 # Package version
 #
-version = mx_version(3, 2, 8)
+version = mx_version(3, 2, 9,
+                     #snapshot=1
+                     )
 
 #
 # Setup information
@@ -132,16 +134,14 @@ distributed under the eGenix.com Public License 1.1.0.
 license = (
 "eGenix.com Public License 1.1.0; "
 "Copyright (c) 1997-2000, Marc-Andre Lemburg, All Rights Reserved; "
-"Copyright (c) 2000-2014, eGenix.com Software GmbH, All Rights Reserved"
+"Copyright (c) 2000-2015, eGenix.com Software GmbH, All Rights Reserved"
 )
 author = "eGenix.com Software GmbH"
 author_email = "info@egenix.com"
 maintainer = "eGenix.com Software GmbH"
 maintainer_email = "info@egenix.com"
 url = "http://www.egenix.com/products/python/mxBase/"
-download_url = 'https://downloads.egenix.com/python/download_url/%s/%s/' % (
-    name,
-    version)
+download_url = url
 platforms = [
     'Windows',
     'Linux',
