@@ -2,7 +2,7 @@
   mxBeeBase -- BeeBase C extension
 
   Copyright (c) 1998-2000, Marc-Andre Lemburg; mailto:mal@lemburg.com
-  Copyright (c) 2000-2014, eGenix.com Software GmbH; mailto:info@egenix.com
+  Copyright (c) 2000-2015, eGenix.com Software GmbH; mailto:info@egenix.com
   See the documentation for further copyright information or contact
   the author (mailto:mal@lemburg.com).
 
@@ -34,11 +34,13 @@
 #include "mxBeeBase.h"
 
 /* Version number: Major.Minor.Patchlevel */
-#define MXBEEBASE_VERSION "3.2.8"
+#define MXBEEBASE_VERSION "3.2.9"
 
 /* Define these to have the module use free lists (saves malloc calls) */
+#ifndef Py_DEBUG
 /*#define MXBEEINDEX_FREELIST*/
 #define MXBEECURSOR_FREELIST
+#endif
 
 /* --- module doc-string -------------------------------------------------- */
 
@@ -47,7 +49,7 @@ static char *Module_docstring =
  MXBEEBASE_MODULE" -- BeeBase objects and functions. Version "MXBEEBASE_VERSION"\n\n"
 
  "Copyright (c) 1998-2000, Marc-Andre Lemburg; mailto:mal@lemburg.com\n"
- "Copyright (c) 2000-2014, eGenix.com Software GmbH; mailto:info@egenix.com\n\n"
+ "Copyright (c) 2000-2015, eGenix.com Software GmbH; mailto:info@egenix.com\n\n"
  "                 All Rights Reserved\n\n"
  "See the documentation for further information on copyrights,\n"
  "or contact the author."
@@ -258,11 +260,17 @@ mxBeeIndexObject *mxBeeIndex_New(char *filename,   /* name of beeindex file */
     mxBeeIndexObject *beeindex = 0;
     bDescription *info;
     bError rc;
-    char *iName = strdup(filename);
+    char *iName;
+    int filename_size;
 
+    /* Copy the filename into iName; we could have used strdup() here, but
+       this code allows us to use pymalloc. */
+    filename_size = strlen(filename) + 1;
+    iName = new(char, filename_size);
     if (iName == NULL)
 	Py_Error(PyExc_MemoryError,
 		 "Out of memory");
+    memcpy(iName, filename, filename_size);
 
     /* Allocate the object */
 #ifdef MXBEEINDEX_FREELIST
