@@ -17,7 +17,7 @@
     Both classes can be subclassed to enhance/modify their behaviour.
 
     Copyright (c) 2000-2005, Marc-Andre Lemburg; mailto:mal@lemburg.com
-    Copyright (c) 2000-2014, eGenix.com Software GmbH; mailto:info@egenix.com
+    Copyright (c) 2000-2015, eGenix.com Software GmbH; mailto:info@egenix.com
     See the documentation for further information on copyrights,
     or contact the author. All Rights Reserved.
 """
@@ -27,6 +27,11 @@ _debug = 0
 import re,sys,types,exceptions
 from mx import TextTools, Tools
 
+### Globals
+
+# Singleton for parameters which were not provided to a call
+NotGiven = 'NotGiven'
+
 ### Errors
 
 class Error(exceptions.StandardError):
@@ -63,11 +68,17 @@ class Reader:
     # Thanks to Andrew Kuchling for helping me with this "simple" RE
     parseitem = re.compile(r'"([^"]|"")*"|[^"' + separators + r'\015\012]*')
 
-    def __init__(self, separators=None):
+    # Encoding; if set to None, no decoding takes place, otherwise all
+    # data is converted to Unicode using this encoding
+    encoding = None
+
+    def __init__(self, separators=None, encoding=NotGiven):
 
         if separators is not None:
             self.parseitem = re.compile(r'"([^"]|"")*"|[^"'
                                         + separators + r'\015\012]*')
+        if encoding is not NotGiven:
+            self.encoding = encoding
 
     def flush(self):
 
@@ -281,7 +292,11 @@ class Reader:
                     # End of line
                     break
                 y = m.regs[0][1]
-                l.append(text[x:y])
+                value = text[x:y]
+                if self.encoding is not None:
+                    # Convert to Unicode, if requested
+                    value = unicode(value, self.encoding)
+                l.append(value)
                 if _debug:
                     print x,repr(l[-1])
                 x = y + 1
@@ -371,12 +386,17 @@ class Writer:
     # End-of-line marker to use (default: CRLF)
     lineend = '\015\012'
 
-    def __init__(self, separator=None, lineend=None):
+    # Encoding to be used for Unicode strings
+    encoding = 'utf-8'
+
+    def __init__(self, separator=None, lineend=None, encoding=None):
 
         if separator is not None:
             self.separator = separator
         if lineend is not None:
             self.lineend = lineend
+        if encoding is not None:
+            self.encoding = encoding
 
     def flush(self):
 
@@ -519,7 +539,7 @@ class Writer:
         for i in rowindices:
             obj = objects[i]
             for j,name in icols:
-                t[i][j] = str(getattr(obj, name))
+                t[i][j] = getattr(obj, name)
                 
         # Quote and join lines
         t = [self.separator.join(self._quote(x)) for x in t]
@@ -555,10 +575,12 @@ class Writer:
         """
         nline = ['""'] * len(line)
         for i,item in Tools.irange(line):
-            if item is not None:
-                text = str(item)
-            else:
+            if item is None:
                 text = ''
+            elif isinstance(item, unicode):
+                text = item.encode(self.encoding)
+            else:
+                text = str(item)
             nline[i] = '"%s"' % text.replace('"','""')
         return nline
 
@@ -620,21 +642,34 @@ newline,new,luck,;123,""\015
         o = Obj()
         o.id = i
         o.str = str(i)
+        o.unicode = unicode(i) + u'\xe4\xf6\xfc'
         data.append(o)
     w.flush()
-    w.set_columns(('id', 'str'))
+    w.set_columns(('id', 'str', 'unicode'))
     w.feed_objects(data)
     w.feed_objects(data)
     s = w.dumps()
     print '-->',s
+    print 'Strings mode:'
     r.flush()
     r.loads(s)
     lt = r.list()
     print lt
     print r.dictionary()
-    r.set_columns(('id','str'))
+    r.set_columns(('id','str', 'unicode'))
     print r.dictionary()
     print r.description()
+    print 'Unicode mode:'
+    r.flush()
+    r.encoding = 'utf-8'
+    r.loads(s)
+    lt = r.list()
+    print lt
+    print r.dictionary()
+    r.set_columns(('id','str', 'unicode'))
+    print r.dictionary()
+    print r.description()
+    r.encoding = None
     print '-' * 72
 
 
