@@ -2,7 +2,7 @@
   mxProxy -- Proxy wrapper type
 
   Copyright (c) 1998-2000, Marc-Andre Lemburg; mailto:mal@lemburg.com
-  Copyright (c) 2000-2014, eGenix.com Software GmbH; mailto:info@egenix.com
+  Copyright (c) 2000-2015, eGenix.com Software GmbH; mailto:info@egenix.com
   See the documentation for further copyright information or contact
   the author (mailto:mal@lemburg.com).
 
@@ -34,10 +34,12 @@
 #include "mxProxy.h"
 
 /* Version number: Major.Minor.Patchlevel */
-#define MXPROXY_VERSION "3.2.8"
+#define MXPROXY_VERSION "3.2.9"
 
 /* Define these to have the module use free lists (saves malloc calls) */
+#ifndef Py_DEBUG
 #define MXPROXY_FREELIST
+#endif
 
 /* --- module doc-string -------------------------------------------------- */
 
@@ -46,7 +48,7 @@ static char *Module_docstring =
  MXPROXY_MODULE" -- Generic proxy wrapper type. Version "MXPROXY_VERSION"\n\n"
 
  "Copyright (c) 1998-2000, Marc-Andre Lemburg; mailto:mal@lemburg.com\n"
- "Copyright (c) 2000-2014, eGenix.com Software GmbH; mailto:info@egenix.com\n\n"
+ "Copyright (c) 2000-2015, eGenix.com Software GmbH; mailto:info@egenix.com\n\n"
  "                 All Rights Reserved\n\n"
  "See the documentation for further information on copyrights,\n"
  "or contact the author."
@@ -673,7 +675,7 @@ mxProxyObject *mxProxy_New(PyObject *object,
 	else if (PySequence_Check(interface)) {
 	    interface = seq2dict(interface);
 	    if (!interface)
-		goto onError;
+		return NULL;
 	}
 	else
 	    Py_Error(PyExc_TypeError,
@@ -694,7 +696,7 @@ mxProxyObject *mxProxy_New(PyObject *object,
 	proxy = PyObject_NEW(mxProxyObject,&mxProxy_Type);
 	if (proxy == NULL) {
 	    Py_XDECREF(interface);
-	    goto onError;
+	    return NULL;
 	}
     }
 
@@ -708,6 +710,8 @@ mxProxyObject *mxProxy_New(PyObject *object,
     else {
 	/* Store the index object instead of the object itself. */
 	if (mxProxy_RegisterWeakReference(proxy, object)) {
+	    /* Make sure we deallocate the object without going through
+	       mxProxy_Free() (as a result of a Py_DECREF). */
 	    mxPy_UNREF(proxy);
 	    PyObject_Del(proxy);
 	    goto onError;
