@@ -2,7 +2,7 @@
   mxQueue -- A queue implementation
 
   Copyright (c) 1999-2000, Marc-Andre Lemburg; mailto:mal@lemburg.com
-  Copyright (c) 2000-2014, eGenix.com Software GmbH; mailto:info@egenix.com
+  Copyright (c) 2000-2015, eGenix.com Software GmbH; mailto:info@egenix.com
   See the documentation for further copyright information or contact
   the author (mailto:mal@lemburg.com).
 
@@ -33,7 +33,7 @@
 #include "mx.h"
 #include "mxQueue.h"
 
-#define MXQUEUE_VERSION "3.2.8"
+#define MXQUEUE_VERSION "3.2.9"
 
 /* The minimal size of queues; must be > 1 */
 #define MINIMAL_QUEUESIZE	4
@@ -54,7 +54,7 @@ static char *Module_docstring =
  MXQUEUE_MODULE" -- A queue implementation. Version "MXQUEUE_VERSION"\n\n"
 
  "Copyright (c) 1999-2000, Marc-Andre Lemburg; mailto:mal@lemburg.com\n"
- "Copyright (c) 2000-2014, eGenix.com Software GmbH; mailto:info@egenix.com\n\n"
+ "Copyright (c) 2000-2015, eGenix.com Software GmbH; mailto:info@egenix.com\n\n"
  "                 All Rights Reserved\n\n"
  "See the documentation for further information on copyrights,\n"
  "or contact the author."
@@ -188,8 +188,10 @@ mxQueueObject *mxQueue_New(Py_ssize_t size)
     /* Init vars */
     if (size < MINIMAL_QUEUESIZE)
 	size = MINIMAL_QUEUESIZE;
+    queue->array = NULL;
     w = new(PyObject*,size);
     if (w == NULL) {
+	Py_DECREF(queue);
 	PyErr_NoMemory();
 	return NULL;
     }
@@ -436,12 +438,12 @@ int mxQueue_Clear(register mxQueueObject *queue)
 static
 mxQueueObject *mxQueue_FromSequence(PyObject *seq)
 {
-    mxQueueObject *queue = 0;
+    mxQueueObject *queue;
     
     /* Create an "empty" queue */
     queue = mxQueue_New(0);
     if (queue == NULL)
-	goto onError;
+	return NULL;
 
     /* Insert items */
     if (mxQueue_PushMany(queue,seq))
@@ -450,8 +452,7 @@ mxQueueObject *mxQueue_FromSequence(PyObject *seq)
     return queue;
 
  onError:
-    if (queue)
-	mxQueue_Free(queue);
+    Py_DECREF(queue);
     return NULL;
 }
 
