@@ -2,7 +2,7 @@
   mxStack -- A stack implementation
 
   Copyright (c) 1997-2000, Marc-Andre Lemburg; mailto:mal@lemburg.com
-  Copyright (c) 2000-2014, eGenix.com Software GmbH; mailto:info@egenix.com
+  Copyright (c) 2000-2015, eGenix.com Software GmbH; mailto:info@egenix.com
   See the documentation for further copyright information or contact
   the author (mailto:mal@lemburg.com).
 
@@ -33,7 +33,7 @@
 #include "mx.h"
 #include "mxStack.h"
 
-#define MXSTACK_VERSION "3.2.8"
+#define MXSTACK_VERSION "3.2.9"
 
 /* The minimal size of stacks; must be > 1 */
 #define MINIMAL_STACKSIZE	4
@@ -54,7 +54,7 @@ static char *Module_docstring =
  MXSTACK_MODULE" -- A stack implementation. Version "MXSTACK_VERSION"\n\n"
 
  "Copyright (c) 1997-2000, Marc-Andre Lemburg; mailto:mal@lemburg.com\n"
- "Copyright (c) 2000-2014, eGenix.com Software GmbH; mailto:info@egenix.com\n\n"
+ "Copyright (c) 2000-2015, eGenix.com Software GmbH; mailto:info@egenix.com\n\n"
  "                 All Rights Reserved\n\n"
  "See the documentation for further information on copyrights,\n"
  "or contact the author."
@@ -188,8 +188,10 @@ mxStackObject *mxStack_New(Py_ssize_t size)
     /* Init vars */
     if (size < MINIMAL_STACKSIZE)
 	size = MINIMAL_STACKSIZE;
+    stack->array = NULL;
     w = new(PyObject*,size);
     if (w == NULL) {
+	Py_DECREF(stack);
 	PyErr_NoMemory();
 	return NULL;
     }
@@ -385,7 +387,7 @@ mxStackObject *mxStack_FromSequence(PyObject *seq)
     /* Create an "empty" stack */
     stack = mxStack_New(0);
     if (stack == NULL)
-	goto onError;
+	return NULL;
 
     /* Insert items */
     if (mxStack_PushMany(stack,seq))
@@ -394,8 +396,7 @@ mxStackObject *mxStack_FromSequence(PyObject *seq)
     return stack;
 
  onError:
-    if (stack)
-	mxStack_Free(stack);
+    Py_DECREF(stack);
     return NULL;
 }
 
