@@ -5,7 +5,7 @@
   8-bit strings and Unicode by setting the TE_* defines appropriately.
 
   Copyright (c) 2000, Marc-Andre Lemburg; mailto:mal@lemburg.com
-  Copyright (c) 2000-2014, eGenix.com Software GmbH; mailto:info@egenix.com
+  Copyright (c) 2000-2015, eGenix.com Software GmbH; mailto:info@egenix.com
 */
 
 
@@ -46,10 +46,26 @@ int TE_HANDLE_MATCH(int flags,
 		    PyObject *context);
 
 /* mxTextTools_TaggingEngine(): a table driven parser engine
-   
-   - return codes: rc = 2: match ok; rc = 1: match failed; rc = 0: error
+
+   Parameters:
+    textobj          - text object to work on
+    text_start       - left text slice index
+    text_stop        - right text slice index
+    table            - tag table object defining the parser
+    taglist          - tag list to append matches to
+    context          - optional context object; may be NULL
+    *next            - output parameter: set to the next index in text
+    level            - stack level; should be 0 on the first level
+  
+   Return codes:
+    rc = 2: match ok
+    rc = 1: match failed
+    rc = 0: error
+
+   Notes:
    - doesn't check type of passed arguments !
    - doesn't increment reference counts of passed objects !
+
 */
 
 int TE_ENGINE_API(PyObject *textobj,
@@ -58,7 +74,8 @@ int TE_ENGINE_API(PyObject *textobj,
 		  mxTagTableObject *table,
 		  PyObject *taglist,
 		  PyObject *context,
-		  Py_ssize_t *next)
+		  Py_ssize_t *next,
+                  int level)
 {
     register Py_ssize_t x;		/* current (head) position in text */
     Py_ssize_t i = 0; 			/* index of current table entry */
@@ -99,6 +116,12 @@ int TE_ENGINE_API(PyObject *textobj,
 		       "invalid slice indexes: [%ld:%ld]",
 		       (long)sliceleft, (long)sliceright);
 
+    /* Protect against stack related segfaults */
+    Py_AssertWithArg(level < Py_GetRecursionLimit(),
+		     PyExc_RuntimeError,
+		     "maximum recursion depth exceeded: %i",
+		     level);
+
     /* Main loop */
     for (i = 0, je = 0;;) {
 	mxTagTableEntry *entry;
@@ -712,7 +735,8 @@ int TE_ENGINE_API(PyObject *textobj,
 		/* match other table */
 		newrc = TE_ENGINE_API(textobj, start, sliceright,
 				      (mxTagTableObject *)match, 
-				      subtags, context, &y);
+				      subtags, context, &y,
+				      level + 1);
 		if (newrc == 0) {
 		    Py_DECREF(subtags);
 		    goto onError;
@@ -850,7 +874,8 @@ int TE_ENGINE_API(PyObject *textobj,
 		/* match other table */
 		newrc = TE_ENGINE_API(textobj, start, sliceright,
 				      (mxTagTableObject *)match, 
-				      subtags, context, &y);
+				      subtags, context, &y,
+		                      level + 1);
 		if (newrc == 0) {
 		    Py_DECREF(subtags);
 		    Py_DECREF(match);
