@@ -2,7 +2,7 @@
   mxTools -- Misc. utilities for Python
 
   Copyright (c) 1997-2000, Marc-Andre Lemburg; mailto:mal@lemburg.com
-  Copyright (c) 2000-2014, eGenix.com Software GmbH; mailto:info@egenix.com
+  Copyright (c) 2000-2015, eGenix.com Software GmbH; mailto:info@egenix.com
   See the documentation for further copyright information or contact
   the author (mailto:mal@lemburg.com).
 
@@ -52,7 +52,7 @@
 # include <dlfcn.h>
 #endif
 
-#define MXTOOLS_VERSION "3.2.8"
+#define MXTOOLS_VERSION "3.2.9"
 
 /* To open up the playground for new ideas... */
 /*#define INCLUDE_FUNSTUFF*/
@@ -67,7 +67,7 @@ static char *Module_docstring =
  MXTOOLS_MODULE" -- A tool collection. Version "MXTOOLS_VERSION"\n\n"
 
  "Copyright (c) 1997-2000, Marc-Andre Lemburg; mailto:mal@lemburg.com\n"
- "Copyright (c) 2000-2014, eGenix.com Software GmbH; mailto:info@egenix.com\n\n"
+ "Copyright (c) 2000-2015, eGenix.com Software GmbH; mailto:info@egenix.com\n\n"
  "                 All Rights Reserved\n\n"
  "See the documentation for further information on copyrights,\n"
  "or contact the author."
@@ -261,10 +261,14 @@ Py_C_Function( mxTools_napply,
     if (w == NULL)
 	goto onError;
 
-    if (arg == NULL)
+    if (arg == NULL) {
 	arg = PyTuple_New(0);
-    if (arg == NULL)
-	goto onError;
+	if (arg == NULL)
+	    goto onError;
+    }
+
+#if 0
+    /* This optimization no longer works... disabled. */
 
 #ifdef PyCFunction_GET_FUNCTION
     /* Short-cut for C functions, taken from ceval.c:call_builtin();
@@ -308,6 +312,7 @@ Py_C_Function( mxTools_napply,
 	}
     }
     else
+#endif
 #endif
 	for (i = 0; i < count; i++) {
 	    register PyObject *v;
@@ -1866,7 +1871,8 @@ Py_C_Function( mxTools_acquire,
 	       "underscore (this too raises an AttributeError).\n")
 {
     PyObject *obj,*baseobj,*name;
-    PyObject *v,*baseobjattr = mxTools_BaseobjAttribute;
+    PyObject *v;
+    PyObject *baseobjattr;
 
     static int recdepth;
 
@@ -1875,6 +1881,14 @@ Py_C_Function( mxTools_acquire,
 	      PyExc_SystemError,
 	      "maximum acquire() recursion depth exceeded");
 
+    /* Set baseobjattr to "baseobj" interned string */
+    if (!mxTools_BaseobjAttribute) {
+	mxTools_BaseobjAttribute = PyString_InternFromString("baseobj");
+	if (!mxTools_BaseobjAttribute)
+	    goto onError;
+    }
+    baseobjattr = mxTools_BaseobjAttribute;
+
     Py_Get3Args("OO|O",
 		obj,name,baseobjattr);
 
@@ -2680,7 +2694,7 @@ PyMethodDef Module_methods[] =
 static 
 void mxToolsModule_Cleanup(void)
 {
-    Py_XDECREF(mxTools_BaseobjAttribute);
+    /* Forget about interned global Python objects */
     mxTools_BaseobjAttribute = 0;
 
     /* Reset mxTools_Initialized flag */
@@ -2697,8 +2711,9 @@ MX_EXPORT(void)
 	Py_Error(PyExc_SystemError,
 		 "can't initialize "MXTOOLS_MODULE" more than once");
 
-    /* Init type objects */
+    /* Init type objects and other globals */
     PyType_Init(mxNotGiven_Type);
+    mxTools_BaseobjAttribute = 0;
 
     /* create module */
     module = Py_InitModule4(MXTOOLS_MODULE, /* Module name */
@@ -2718,11 +2733,6 @@ MX_EXPORT(void)
     if (!mxNotGiven)
 	goto onError;
 
-    /* Create acquire default argument for baseobjattr */
-    mxTools_BaseobjAttribute = PyString_InternFromString("baseobj");
-    if (!mxTools_BaseobjAttribute)
-	goto onError;
-
     /* Add some constants to the module's dict */
     moddict = PyModule_GetDict(module);
     insstr(moddict, "__version__", MXTOOLS_VERSION);
