@@ -2,7 +2,7 @@
   mxURL -- An URL datatype
 
   Copyright (c) 1998-2000, Marc-Andre Lemburg; mailto:mal@lemburg.com
-  Copyright (c) 2000-2014, eGenix.com Software GmbH; mailto:info@egenix.com
+  Copyright (c) 2000-2015, eGenix.com Software GmbH; mailto:info@egenix.com
   See the documentation for further copyright information or contact
   the author (mailto:mal@lemburg.com).
 
@@ -36,10 +36,12 @@
 #include <ctype.h>
 
 /* Version number: Major.Minor.Patchlevel */
-#define MXURL_VERSION "3.2.8"
+#define MXURL_VERSION "3.2.9"
 
 /* Define this to have the module use a free list for URLs */
+#ifndef Py_DEBUG
 #define MXURL_FREELIST
+#endif
 
 /* Define this to enable the speedup in mxURL_SchemeUsesRelativePaths()
    that uses hard-coded values for the uses_relative part of the
@@ -56,7 +58,7 @@ static char *Module_docstring =
  "Version "MXURL_VERSION"\n\n"
 
  "Copyright (c) 1998-2000, Marc-Andre Lemburg; mailto:mal@lemburg.com\n"
- "Copyright (c) 2000-2014, eGenix.com Software GmbH; mailto:info@egenix.com\n\n"
+ "Copyright (c) 2000-2015, eGenix.com Software GmbH; mailto:info@egenix.com\n\n"
  "                 All Rights Reserved\n\n"
  "See the documentation for further information on copyrights,\n"
  "or contact the author."
@@ -1376,7 +1378,7 @@ mxURLObject *mxURL_FromString(char *str,
     
     url = mxURL_New();
     if (!url)
-	goto onError;
+	return NULL;
 
     /* Init vars */
     if (mxURL_SetFromString(url,str,normalize) < 0)
@@ -1384,8 +1386,7 @@ mxURLObject *mxURL_FromString(char *str,
     return url;
 
  onError:
-    if (url)
-	mxURL_Free(url);
+    Py_XDECREF(url);
     return NULL;
 }
 
@@ -1402,7 +1403,7 @@ mxURLObject *mxURL_FromBrokenDown(char *scheme,
     
     url = mxURL_New();
     if (!url)
-	goto onError;
+	return NULL;
 
     /* Init vars */
     if (mxURL_SetFromBrokenDown(url,
@@ -1423,8 +1424,7 @@ mxURLObject *mxURL_FromBrokenDown(char *scheme,
     return url;
 
  onError:
-    if (url)
-	mxURL_Free(url);
+    Py_XDECREF(url);
     return NULL;
 }
 
@@ -1448,7 +1448,7 @@ mxURLObject *mxURL_RebuildFromBrokenDown(mxURLObject *url,
 
     newurl = mxURL_New();
     if (!newurl)
-	goto onError;
+	return NULL;
     
     if (!scheme && url->scheme) {
 	scheme = PyString_AS_STRING(url->scheme);
@@ -1511,8 +1511,7 @@ mxURLObject *mxURL_RebuildFromBrokenDown(mxURLObject *url,
     return newurl;
 
  onError:
-    if (newurl)
-	mxURL_Free(newurl);
+    Py_XDECREF(newurl);
     return NULL;
 }
 
@@ -1544,6 +1543,7 @@ mxURLObject *mxURL_NormalizedFromURL(mxURLObject *url)
     mxURLObject *normurl = 0;
     char *str = PyString_AS_STRING(url->url);
 
+    /* Shortcut */
     if (url->path_normalized) {
 	Py_INCREF(url);
 	return url;
@@ -1551,7 +1551,7 @@ mxURLObject *mxURL_NormalizedFromURL(mxURLObject *url)
 
     normurl = mxURL_New();
     if (!normurl)
-	goto onError;
+	return NULL;
 
     /* Init vars */
     if (mxURL_SetFromBrokenDown(normurl,
@@ -1574,8 +1574,7 @@ mxURLObject *mxURL_NormalizedFromURL(mxURLObject *url)
     return normurl;
 
  onError:
-    if (normurl)
-	mxURL_Free(normurl);
+    Py_XDECREF(normurl);
     return NULL;
 }
 
@@ -1600,7 +1599,7 @@ mxURLObject *mxURL_BasicFromURL(mxURLObject *url)
 
     basicurl = mxURL_New();
     if (!basicurl)
-	goto onError;
+	return NULL;
 
     /* Init vars */
     if (mxURL_SetFromBrokenDown(basicurl,
@@ -1620,8 +1619,7 @@ mxURLObject *mxURL_BasicFromURL(mxURLObject *url)
     return basicurl;
 
  onError:
-    if (basicurl)
-	mxURL_Free(basicurl);
+    Py_XDECREF(basicurl);
     return NULL;
 }
 
@@ -1755,8 +1753,7 @@ mxURLObject *mxURL_RelativeFromURL(mxURLObject *url,
  onError:
     if (relpath)
 	free(relpath);
-    if (relurl)
-	mxURL_Free(relurl);
+    Py_XDECREF(relurl);
     return NULL;
 }
 
@@ -2805,8 +2802,11 @@ void mxURLModule_Cleanup(void)
     }
 #endif
     if (!mxPy_DECREF_Unsafe_AtExit()) {
-	Py_XDECREF(mxURL_MIMEDict);
+	/* We keep a weakref global mxURL_MIMEDict, but this lives in the
+	   mimetools module dictionary and gets GCed there. */
     }
+
+    /* Clear weakref globals */
     mxURL_MIMEDict = NULL;
 
     /* Reset mxURL_Initialized flag */
